<?php
/**
 * This class handles the exports made in the "Statistics" menu.
 *
 * @package helpful-pro
 */

/**
 * This class handles the exports made in the "Statistics" menu.
 */
class Daexthefup_Export_Controller {

	/**
	 * Store an instance of the shared class.
	 *
	 * @var Daln_Shared|null
	 */
	private $shared = null;

	public function __construct( $shared ) {

		// assign an instance of the plugin info.
		$this->shared = $shared;

		// Export XML controller.
		add_action( 'init', array( $this, 'export_csv_controller' ) );
	}

	/**
	 * The "Export CSV" buttons and/or icons available in the Dashboard, Juice
	 * and Hits menus are intercepted and the proper method that generates on
	 * the fly the specific downloadable CSV file is called.
	 *
	 * @return void
	 */
	public function export_csv_controller() {

		/**
		 * Intercept requests that come from the "Export CSV" button from the "Statistics" menu and generate the
		 * downloadable CSV file with the statistics_menu_export_csv() method.
		 */
		if ( isset( $_GET['page'] ) and
			isset( $_POST['export_csv'] ) ) {
			$page        = sanitize_key( $_GET['page'] );
			$export_type = intval( $_POST['export_csv_export_type'], 10 );
			if ( $page === 'daexthefup-statistics' ) {
				$this->statistics_menu_export_csv( $export_type );
			}
		}

		/**
		 * Intercept requests that come from the download icon associated with the URLs of the "Statistics" menu and
		 * generate the downloadable CSV file with the statistics_menu_export_post_feedback_csv() method.
		 */
		if ( isset( $_GET['page'] ) && isset( $_POST['post_feedback_csv'] ) && isset( $_POST['post_id'] ) ) {
			$page = sanitize_key( $_GET['page'] );
			if ( $page === 'daexthefup-statistics' ) {
				$this->statistics_menu_export_post_feedback_csv();
			}
		}
	}

	/**
	 * Generates the downloadable CSV file with all the feedback associated with a specific post listed in the
	 * "Statistics" menu.
	 *
	 * @return void
	 */
	private function statistics_menu_export_post_feedback_csv() {

		// verify capability.
		if ( ! current_user_can( get_option( $this->shared->get( 'slug' ) . '_statistics_menu_capability' ) ) ) {
			die();}

		// Set the PHP "Max Execution Time" and "Memory Limit" based on the values defined in the options.
		$this->shared->set_met_and_ml();

		// get the Post ID.
		$post_id = intval( $_POST['post_id'], 10 );

		// get the data from the db table.
		global $wpdb;
		$table_name = $wpdb->prefix . $this->shared->get( 'slug' ) . '_feedback';
		$safe_sql   = $wpdb->prepare( "SELECT * FROM $table_name WHERE post_id = %d ORDER BY feedback_id DESC", $post_id );
		$results    = $wpdb->get_results( $safe_sql, ARRAY_A );

		$csv_content = $this->get_itemized_feedback_csv_content( $results, 'itemized-feedback-post-' . $post_id );

		echo $csv_content;
		die();
	}

	/**
	 * Generates the downloadable CSV file with all the posts listed in the "Statistics" menu.
	 *
	 * @param $export_type
	 *
	 * @return void
	 */
	private function statistics_menu_export_csv( $export_type ) {

		// verify capability.
		if ( ! current_user_can( get_option( $this->shared->get( 'slug' ) . '_statistics_menu_capability' ) ) ) {
			die();}

		// Set the PHP "Max Execution Time" and "Memory Limit" based on the values defined in the options.
		$this->shared->set_met_and_ml();

		if ( 0 === $export_type ) {

			// Cumulative  ------------------------------------------------------------------------------------------------.

			// get the data from the db table.
			global $wpdb;
			$table_name = $wpdb->prefix . $this->shared->get( 'slug' ) . '_archive';
			$safe_sql   = "SELECT * FROM $table_name ORDER BY archive_id DESC";
			$results    = $wpdb->get_results( $safe_sql, ARRAY_A );

			$csv_content = $this->get_cumulalative_feedback_csv_content( $results );

		} else {

			// Itemized -----------------------------------------------------------------------------------------------.

			// get the data from the db table.
			global $wpdb;
			$table_name = $wpdb->prefix . $this->shared->get( 'slug' ) . '_feedback';
			$safe_sql   = "SELECT * FROM $table_name ORDER BY feedback_id DESC";
			$results    = $wpdb->get_results( $safe_sql, ARRAY_A );

			$csv_content = $this->get_itemized_feedback_csv_content( $results, 'itemized-feedback' );

		}

		echo $csv_content;
		die();
	}

	/**
	 * Escape the double quotes of the $content string, so the returned string
	 * can be used in CSV fields enclosed by double quotes.
	 *
	 * @param $content The unescape content ( Ex: He said "Yes!" )
	 * @param $content
	 *
	 * @return string The escaped content ( Ex: He said ""Yes!"" )
	 *
	 * @return array|string|string[]
	 */
	private function esc_csv( $content ) {
		return str_replace( '"', '""', $content );
	}

	/**
	 * Generate the CSV content for the "Cumulative Feedback" export.
	 *
	 * @param $results Array of records of the "feedback" table
	 *
	 * @return false|string
	 */
	private function get_cumulalative_feedback_csv_content( $results ) {

		// if there are data generate the csv header and content.
		if ( count( $results ) > 0 ) {

			$csv_content = '';
			$new_line    = "\n";

			// set the csv header.
			header( 'Content-Encoding: UTF-8' );
			header( 'Content-type: text/csv; charset=UTF-8' );
			header( 'Content-Disposition: attachment; filename=cumulative-feedback-' . time() . '.csv' );
			header( 'Pragma: no-cache' );
			header( 'Expires: 0' );

			// set headings.
			$csv_content .= '"' . $this->esc_csv( __( 'Post Title', 'helpful-pro' ) ) . '",';
			$csv_content .= '"' . $this->esc_csv( __( 'Post Type', 'helpful-pro' ) ) . '",';
			$csv_content .= '"' . $this->esc_csv( __( 'Post Date', 'helpful-pro' ) ) . '",';
			$csv_content .= '"' . $this->esc_csv( __( 'Positive Feedback', 'helpful-pro' ) ) . '",';
			$csv_content .= '"' . $this->esc_csv( __( 'Negative Feedback', 'helpful-pro' ) ) . '",';
			$csv_content .= '"' . $this->esc_csv( __( 'PFR', 'helpful-pro' ) ) . '"';

			$csv_content .= $new_line;

			// set column content.
			foreach ( $results as $result ) {

				$csv_content .= '"' . $this->esc_csv( $result['post_title'] ) . '",';
				$csv_content .= '"' . $this->esc_csv( $result['post_type'] ) . '",';
				$csv_content .= '"' . $this->esc_csv( $result['post_date'] ) . '",';
				$csv_content .= '"' . $this->esc_csv( $result['positive_feedback'] ) . '",';
				$csv_content .= '"' . $this->esc_csv( $result['negative_feedback'] ) . '",';
				$csv_content .= '"' . $this->esc_csv( $result['pfr'] !== '-1' ? $result['pfr'] . '%' : 'N/A' ) . '"';

				$csv_content .= $new_line;

			}
		} else {
			return false;
		}

		return $csv_content;
	}

	/**
	 * Generates the CSV content for the "Itemized Feedback" export.
	 *
	 * @param $results Array of records of the "feedback" table
	 * @param $report_name A string with the name of the report. It will be used to generate the name of the CSV file
	 *
	 * @return false|string
	 */
	private function get_itemized_feedback_csv_content( $results, $report_name ) {

		// if there are data generate the csv header and content.
		if ( count( $results ) > 0 ) {

			$csv_content = '';
			$new_line    = "\n";

			// set the csv header.
			header( 'Content-Encoding: UTF-8' );
			header( 'Content-type: text/csv; charset=UTF-8' );
			header( 'Content-Disposition: attachment; filename=' . $report_name . '-' . time() . '.csv' );
			header( 'Pragma: no-cache' );
			header( 'Expires: 0' );

			// set headings.
			$csv_content .= '"' . $this->esc_csv( __( 'Post', 'helpful-pro' ) ) . '",';
			$csv_content .= '"' . $this->esc_csv( __( 'Date', 'helpful-pro' ) ) . '",';
			$csv_content .= '"' . $this->esc_csv( __( 'Rating', 'helpful-pro' ) ) . '",';
			$csv_content .= '"' . $this->esc_csv( __( 'Content', 'helpful-pro' ) ) . '",';

			$csv_content .= $new_line;

			// set column content.
			foreach ( $results as $result ) {

				$csv_content .= '"' . $this->esc_csv( get_the_title( $result['post_id'] ) ) . '",';
				$csv_content .= '"' . $this->esc_csv( $result['date'] ) . '",';
				$csv_content .= '"' . $this->esc_csv( intval( $result['value'], 10 ) === 0 ? __( 'Negative', 'helpful-pro' ) : __( 'Positive', 'helpful-pro' ) ) . '",';
				$csv_content .= '"' . $this->esc_csv( stripslashes( $result['description'] ) ) . '",';

				$csv_content .= $new_line;

			}
		} else {
			return false;
		}

		return $csv_content;
	}
}
